﻿using EVCB_OCPP.Packet.Messages;
using EVCB_OCPP.Packet.Messages.Core;
using EVCB_OCPP.Packet.Messages.FirmwareManagement;
using EVCB_OCPP.Packet.Messages.LocalAuthListManagement;
using EVCB_OCPP.Packet.Messages.RemoteTrigger;
using EVCB_OCPP.Packet.Messages.Reservation;
using EVCB_OCPP.Packet.Messages.SmartCharging;
using EVCB_OCPP.Packet.Messages.SubTypes;
using EVCB_OCPP.WEBAPI.Handlers;
using EVCB_OCPP.WEBAPI.Models.WebAPI;
using EVCB_OCPP.WEBAPI.Services;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web.Http;
using System.Web.Http.Description;
using StartTransactionRequest = EVCB_OCPP.WEBAPI.Models.WebAPI.StartTransactionRequest;

namespace EVCB_OCPP.WEBAPI.Controllers
{
    [RoutePrefix("api/v1/ocpp16")]
    //[InernalAuthentication]
    public class InternalV1Controller : ApiController
    {

        [Route("command/{OrderNo}/SerialNo/{SerialNo}/chargebox/{ChargeBoxId}")]
        [ResponseType(typeof(ErrorResponse))]
        [ResponseType(typeof(InternalGenericResponse))]
        [HttpGet]
        public HttpResponseMessage CommandofExecution(int OrderNo, string SerialNo, string ChargeBoxId)
        {
            var result = new InternalGenericResponse();
            HttpStatusCode statusCode = HttpStatusCode.InternalServerError;
            try
            {

                ChargePointService chargePointService = new ChargePointService();

                if (!IsValidEnum<Internal_Actions>(OrderNo) || string.IsNullOrEmpty(ChargeBoxId))
                {
                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_PARAMETER_OUTOFRANGE_INCORRECT });
                }

                if (!chargePointService.Exists(ChargeBoxId))
                {
                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CHARGEBOXID_ISNT_EXIST_OR_OFFLINE });
                }

                ServerTriggerService triggerService = new ServerTriggerService();
                var excution = triggerService.GetExecution(SerialNo, (Internal_Actions)OrderNo, ChargeBoxId);

                if (excution == null)
                {
                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_DATA_NOTFOUND });
                }

                result.Message = JsonConvert.SerializeObject(excution, EVCBlConfiguration.JSONSERIALIZER_FORMAT);
                statusCode = HttpStatusCode.OK;

            }
            catch (Exception ex)
            {

                return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse()
                {
                    Message = string.Format(EVCBlConfiguration.ERRORMESSAGE_FORMAT, EVCBlConfiguration.ERROR_MSG_UNEXPECTEDERROR, "CommandofExecution", ex.ToString())
                });
            }

            return Request.CreateResponse(statusCode, result);
        }

        [Route("availability/chargebox/{ChargeBoxId}")]
        [ResponseType(typeof(ErrorResponse))]
        [ResponseType(typeof(InternalGenericResponse))]
        [HttpPut]
        public HttpResponseMessage Availability(string ChargeBoxId, [FromBody]AvailiabilityRequest Availiability)
        {
            string uuid = Guid.NewGuid().ToString();
            var result = new InternalGenericResponse();
            HttpStatusCode statusCode = HttpStatusCode.InternalServerError;
            try
            {

                ChargePointService chargePointService = new ChargePointService();

                if (!chargePointService.IsOnline(ChargeBoxId))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CHARGEBOXID_ISNT_EXIST_OR_OFFLINE });
                }

                //check parameter
                if (Availiability == null ||
                    !(Availiability.ConnectorId >= 0) ||
                    !(Availiability.AvailabilityType > 0 && Availiability.AvailabilityType < 3))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_PARAMETER_OUTOFRANGE_INCORRECT });
                }

                //create reqest

                var request = new ChangeAvailabilityRequest()
                {
                    connectorId = Availiability.ConnectorId,
                    type = (EVCB_OCPP.Packet.Messages.SubTypes.AvailabilityType)Availiability.AvailabilityType
                };


                if (!AddCommandtoServer(ChargeBoxId, uuid, request))
                {
                    throw new Exception("Write Command Fail!!");
                }

                statusCode = HttpStatusCode.Accepted;
                result.SerialNo = uuid;

            }
            catch (Exception ex)
            {

                return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse()
                {
                    Message = string.Format(EVCBlConfiguration.ERRORMESSAGE_FORMAT, EVCBlConfiguration.ERROR_MSG_UNEXPECTEDERROR, "Availability", ex.ToString())
                });
            }

            return Request.CreateResponse(statusCode, result);
        }


        [Route("configuration/chargebox/{ChargeBoxId}")]
        [ResponseType(typeof(ErrorResponse))]
        [ResponseType(typeof(InternalGenericResponse))]
        [HttpPut]
        public HttpResponseMessage ChargeBoxConfiguration(string ChargeBoxId, [FromBody]SingleConfigurationRequest SingleConfiguration)
        {
            string uuid = Guid.NewGuid().ToString();
            var result = new InternalGenericResponse();
            HttpStatusCode statusCode = HttpStatusCode.InternalServerError;
            try
            {
                //Does charge exist ? 
                ChargePointService chargePointService = new ChargePointService();


                if (!chargePointService.IsOnline(ChargeBoxId))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CHARGEBOXID_ISNT_EXIST_OR_OFFLINE });
                }

                //check parameter
                if (SingleConfiguration == null ||
                    (string.IsNullOrEmpty(SingleConfiguration.Key) || string.IsNullOrEmpty(SingleConfiguration.Value)))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_PARAMETER_OUTOFRANGE_INCORRECT });
                }

                //create reqest
                var request = new ChangeConfigurationRequest()
                {
                    key = SingleConfiguration.Key,
                    value = SingleConfiguration.Value
                };


                if (!AddCommandtoServer(ChargeBoxId, uuid, request))
                {
                    throw new Exception("Write Command Fail!!");
                }

                statusCode = HttpStatusCode.Accepted;
                result.SerialNo = uuid;
            }
            catch (Exception ex)
            {

                return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse()
                {
                    Message = string.Format(EVCBlConfiguration.ERRORMESSAGE_FORMAT, EVCBlConfiguration.ERROR_MSG_UNEXPECTEDERROR, "ChangeConfiguration", ex.ToString())
                });
            }

            return Request.CreateResponse(statusCode, result);
        }

        [Route("configuration/chargebox/{ChargeBoxId}")]
        [ResponseType(typeof(ErrorResponse))]
        [ResponseType(typeof(InternalGenericResponse))]
        [HttpGet]
        public HttpResponseMessage ChargeBoxConfiguration(string ChargeBoxId, string Keys)
        {
            string uuid = Guid.NewGuid().ToString();
            var result = new InternalGenericResponse();
            HttpStatusCode statusCode = HttpStatusCode.InternalServerError;
            try
            {
                //Does charge exist ? 
                ChargePointService chargePointService = new ChargePointService();

                if (!chargePointService.IsOnline(ChargeBoxId))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CHARGEBOXID_ISNT_EXIST_OR_OFFLINE });
                }

                //check parameter
                if (string.IsNullOrEmpty(Keys) || Keys.Split('/').Count() > 20)
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_PARAMETER_OUTOFRANGE_INCORRECT });
                }

                //create reqest
                var request = new GetConfigurationRequest()
                {
                    key = Keys.Split('/').ToList()
                };


                if (!AddCommandtoServer(ChargeBoxId, uuid, request))
                {
                    throw new Exception("Write Command Fail!!");
                }

                statusCode = HttpStatusCode.OK;
                result.SerialNo = uuid;
            }
            catch (Exception ex)
            {

                return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse()
                {
                    Message = string.Format(EVCBlConfiguration.ERRORMESSAGE_FORMAT, EVCBlConfiguration.ERROR_MSG_UNEXPECTEDERROR, "GetConfiguration", ex.ToString())
                });
            }

            return Request.CreateResponse(statusCode, result);
        }

        [Route("diagonostics/chargebox/{ChargeBoxId}")]
        [ResponseType(typeof(ErrorResponse))]
        [ResponseType(typeof(InternalGenericResponse))]
        [HttpGet]
        public HttpResponseMessage Diagonostics(string ChargeBoxId, Uri Location, int Retries = -1, int RetryInterval = -1, string StartTime = null, string StopTime = null)
        {
            string uuid = Guid.NewGuid().ToString();
            var result = new InternalGenericResponse();
            HttpStatusCode statusCode = HttpStatusCode.InternalServerError;
            try
            {
                //Does charge exist ? 
                ChargePointService chargePointService = new ChargePointService();


                if (!chargePointService.IsOnline(ChargeBoxId))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CHARGEBOXID_ISNT_EXIST_OR_OFFLINE });
                }

                //check parameter
                DateTime vaildtime = new DateTime();
                if (!Location.IsWellFormedOriginalString()
                    || (!string.IsNullOrEmpty(StartTime) && !DateTime.TryParse(StartTime, out vaildtime))
                    || (!string.IsNullOrEmpty(StopTime) && !DateTime.TryParse(StopTime, out vaildtime)))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_PARAMETER_OUTOFRANGE_INCORRECT });
                }

                //create reqest
                var request = new GetDiagnosticsRequest()
                {
                    location = Location,
                    retries = Retries == -1 ? (int?)null : Retries,
                    retryInterval = RetryInterval == -1 ? (int?)null : RetryInterval,
                    startTime = StartTime == null ? (DateTime?)null : DateTime.SpecifyKind(DateTime.Parse(StartTime), DateTimeKind.Utc),
                    stopTime = StopTime == null ? (DateTime?)null : DateTime.SpecifyKind(DateTime.Parse(StopTime), DateTimeKind.Utc)

                };

                if (!AddCommandtoServer(ChargeBoxId, uuid, request))
                {
                    throw new Exception("Write Command Fail!!");
                }

                statusCode = HttpStatusCode.Accepted;
                result.SerialNo = uuid;

            }
            catch (Exception ex)
            {

                return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse()
                {
                    Message = string.Format(EVCBlConfiguration.ERRORMESSAGE_FORMAT, EVCBlConfiguration.ERROR_MSG_UNEXPECTEDERROR, "GetDiagonostics", ex.ToString())
                });
            }

            return Request.CreateResponse(statusCode, result);
        }


        [Route("locallistversion/chargebox/{ChargeBoxId}")]
        [ResponseType(typeof(ErrorResponse))]
        [ResponseType(typeof(InternalGenericResponse))]
        [HttpGet]
        public HttpResponseMessage Locallistversion(string ChargeBoxId)
        {
            string uuid = Guid.NewGuid().ToString();
            var result = new InternalGenericResponse();
            HttpStatusCode statusCode = HttpStatusCode.InternalServerError;
            try
            {

                ChargePointService chargePointService = new ChargePointService();

                if (!chargePointService.IsOnline(ChargeBoxId))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CHARGEBOXID_ISNT_EXIST_OR_OFFLINE });
                }

                if (!AddCommandtoServer(ChargeBoxId, uuid, new GetLocalListVersionRequest()))
                {
                    throw new Exception("Write Command Fail!!");
                }

                statusCode = HttpStatusCode.Accepted;
                result.SerialNo = uuid;

            }
            catch (Exception ex)
            {

                return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = string.Format(EVCBlConfiguration.ERRORMESSAGE_FORMAT, EVCBlConfiguration.ERROR_MSG_UNEXPECTEDERROR, "GetLocallistversion", ex.ToString()) });
            }

            return Request.CreateResponse(statusCode, result);
        }

        [Route("transaction/chargebox/{ChargeBoxId}")]
        [ResponseType(typeof(ErrorResponse))]
        [ResponseType(typeof(InternalGenericResponse))]
        [HttpPost]
        public HttpResponseMessage StartTransaction(string ChargeBoxId, [FromBody]StartTransactionRequest StartTransaction)
        {
            string uuid = Guid.NewGuid().ToString();
            var result = new InternalGenericResponse();
            HttpStatusCode statusCode = HttpStatusCode.InternalServerError;
            try
            {
                ChargePointService chargePointService = new ChargePointService();

                if (!chargePointService.IsOnline(ChargeBoxId))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CHARGEBOXID_ISNT_EXIST_OR_OFFLINE });
                }

                //check parameter
                if (StartTransaction == null || string.IsNullOrEmpty(StartTransaction.IdTag) ||
                    !(StartTransaction.ConnectorId.HasValue && StartTransaction.ConnectorId > 0) ||
                    !(StartTransaction.ChargingProfile != null && IsValidEnum<ChargingProfileKindType>((int)StartTransaction.ChargingProfile.chargingProfileKind)) ||
                    !(StartTransaction.ChargingProfile != null && StartTransaction.ChargingProfile.recurrencyKind.HasValue && IsValidEnum<RecurrencyKindType>((int)StartTransaction.ChargingProfile.recurrencyKind)) ||
                    !(StartTransaction.ChargingProfile != null && StartTransaction.ChargingProfile.recurrencyKind.HasValue && IsValidEnum<RecurrencyKindType>((int)StartTransaction.ChargingProfile.recurrencyKind)) ||
                     !(StartTransaction.ChargingProfile != null && StartTransaction.ChargingProfile.chargingSchedule != null && IsValidEnum<ChargingRateUnitType>((int)StartTransaction.ChargingProfile.chargingSchedule.chargingRateUnit)))

                {
                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_PARAMETER_OUTOFRANGE_INCORRECT });
                }

                if (StartTransaction.ConnectorId.HasValue && chargePointService.GetChargePointCurrentSatus(ChargeBoxId, StartTransaction.ConnectorId.Value).HasValue &&
                    chargePointService.GetChargePointCurrentSatus(ChargeBoxId, StartTransaction.ConnectorId.Value).Value != ChargePointStatus.Available)
                {
                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CONNECTOR_ISNOT_AVAILIABLE_MODE });
                }
                else
                {
                    var _request = new RemoteStartTransactionRequest()
                    {
                        connectorId = StartTransaction.ConnectorId,
                        idTag = StartTransaction.IdTag,
                        chargingProfile = StartTransaction.ChargingProfile == null ? null :
                         new csChargingProfiles()
                         {
                             chargingProfileId = StartTransaction.ChargingProfile.chargingProfileId,
                             stackLevel = StartTransaction.ChargingProfile.stackLevel,
                             chargingProfilePurpose = ChargingProfilePurposeType.TxProfile,
                             chargingProfileKind = StartTransaction.ChargingProfile.chargingProfileKind,
                             recurrencyKind = StartTransaction.ChargingProfile.recurrencyKind,
                             validFrom = StartTransaction.ChargingProfile.validFrom == null ? (DateTime?)null : DateTime.SpecifyKind(StartTransaction.ChargingProfile.validFrom.Value, DateTimeKind.Utc),
                             validTo = StartTransaction.ChargingProfile.validTo == null ? (DateTime?)null : DateTime.SpecifyKind(StartTransaction.ChargingProfile.validTo.Value, DateTimeKind.Utc),
                             chargingSchedule = new ChargingSchedule()
                             {
                                 chargingRateUnit = StartTransaction.ChargingProfile.chargingSchedule.chargingRateUnit,
                                 chargingSchedulePeriod = StartTransaction.ChargingProfile.chargingSchedule.chargingSchedulePeriod,
                                 duration = StartTransaction.ChargingProfile.chargingSchedule.duration,
                                 minChargingRate = StartTransaction.ChargingProfile.chargingSchedule.minChargingRate,
                                 startSchedule = StartTransaction.ChargingProfile.chargingSchedule.startSchedule == null ? (DateTime?)null : DateTime.SpecifyKind(StartTransaction.ChargingProfile.chargingSchedule.startSchedule.Value, DateTimeKind.Utc)
                             }
                         }

                    };


                    if (!AddCommandtoServer(ChargeBoxId, uuid, _request))
                    {
                        throw new Exception("Write Command Fail!!");
                    }
                }

                statusCode = HttpStatusCode.Accepted;
                result.SerialNo = uuid;

            }
            catch (Exception ex)
            {

                return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse()
                {
                    Message = string.Format(EVCBlConfiguration.ERRORMESSAGE_FORMAT, EVCBlConfiguration.ERROR_MSG_UNEXPECTEDERROR, "StopTransaction", ex.ToString())
                });
            }

            return Request.CreateResponse(statusCode, result);
        }

        [Route("transaction/{TransactionId}/chargebox/{ChargeBoxId}")]
        [ResponseType(typeof(ErrorResponse))]
        [ResponseType(typeof(InternalGenericResponse))]
        [HttpPut]
        public HttpResponseMessage StopTransaction(string ChargeBoxId, int TransactionId)
        {
            string uuid = Guid.NewGuid().ToString();
            var result = new InternalGenericResponse();
            HttpStatusCode statusCode = HttpStatusCode.InternalServerError;
            try
            {
                ChargePointService chargePointService = new ChargePointService();

                if (!chargePointService.IsOnline(ChargeBoxId))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CHARGEBOXID_ISNT_EXIST_OR_OFFLINE });
                }

                if (!chargePointService.GetChargePointCurrentSatus(ChargeBoxId, TransactionId % 10).HasValue)
                {
                    //offline
                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CONNECTOR_ISNOT_AVAILIABLE_MODE });
                }
                else
                {
                    if (chargePointService.IsTransactionRunning(ChargeBoxId, TransactionId))
                    {
                        var _request = new RemoteStopTransactionRequest()
                        {
                            transactionId = TransactionId
                        };

                        //下發停止
                        if (!AddCommandtoServer(ChargeBoxId, uuid, _request))
                        {
                            throw new Exception("Write Command Fail!!");
                        }
                    }
                    else
                    {
                        return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_SESSION_WAS_FINISHED });
                    }
                }

                statusCode = HttpStatusCode.Accepted;
                result.SerialNo = uuid;

            }
            catch (Exception ex)
            {

                return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse()
                {
                    Message = string.Format(EVCBlConfiguration.ERRORMESSAGE_FORMAT, EVCBlConfiguration.ERROR_MSG_UNEXPECTEDERROR, "StopTransaction", ex.ToString())
                });
            }

            return Request.CreateResponse(statusCode, result);
        }

        [Route("reservation/chargebox/{ChargeBoxId}")]
        [ResponseType(typeof(ErrorResponse))]
        [ResponseType(typeof(InternalGenericResponse))]
        [HttpPost]
        public HttpResponseMessage ReserveNow(string ChargeBoxId, [FromBody] ReserveRequest ReserveNow)
        {
            string uuid = Guid.NewGuid().ToString();
            var result = new InternalGenericResponse();
            HttpStatusCode statusCode = HttpStatusCode.InternalServerError;
            try
            {
                ChargePointService chargePointService = new ChargePointService();

                if (!chargePointService.IsOnline(ChargeBoxId))
                {
                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CHARGEBOXID_ISNT_EXIST_OR_OFFLINE });
                }


                DateTime expiryDate = new DateTime();
                //check parameter
                if (ReserveNow == null ||
                    !(ReserveNow.ConnectorId >= 0) ||
                    string.IsNullOrEmpty(ReserveNow.IdTag) ||
                    ReserveNow.ReservationId == -1 ||
                    !DateTime.TryParse(ReserveNow.ExpiryDate, out expiryDate))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_PARAMETER_OUTOFRANGE_INCORRECT });
                }

                expiryDate = DateTime.SpecifyKind(DateTime.Parse(ReserveNow.ExpiryDate), DateTimeKind.Utc);

                if (DateTime.Compare(DateTime.UtcNow, expiryDate) >= 0)
                {
                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_PARAMETER_OUTOFRANGE_INCORRECT });
                }

                var connectorStatus = chargePointService.GetChargePointCurrentSatus(ChargeBoxId, ReserveNow.ConnectorId);

                if (connectorStatus.HasValue && connectorStatus.Value == ChargePointStatus.Available)
                {
                    //create reqest

                    var request = new ReserveNowRequest()
                    {
                        connectorId = ReserveNow.ConnectorId,
                        expiryDate = expiryDate,
                        idTag = ReserveNow.IdTag,
                        reservationId = ReserveNow.ReservationId,
                        parentIdTag = ReserveNow.ParentIdTag

                    };


                    if (!AddCommandtoServer(ChargeBoxId, uuid, request))
                    {
                        throw new Exception("Write Command Fail!!");
                    }

                    statusCode = HttpStatusCode.Accepted;
                    result.SerialNo = uuid;
                }
                else
                {
                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CONNECTOR_ISNOT_AVAILIABLE_MODE });
                }



            }
            catch (Exception ex)
            {

                return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse()
                {
                    Message = string.Format(EVCBlConfiguration.ERRORMESSAGE_FORMAT, EVCBlConfiguration.ERROR_MSG_UNEXPECTEDERROR, "ReserveNow", ex.ToString())
                });
            }

            return Request.CreateResponse(statusCode, result);
        }

        [Route("reservation/{ReservationId}/chargebox/{ChargeBoxId}")]
        [ResponseType(typeof(ErrorResponse))]
        [ResponseType(typeof(InternalGenericResponse))]
        [HttpDelete]
        public HttpResponseMessage CancelResrvation(string ChargeBoxId, int ReservationId)
        {
            string uuid = Guid.NewGuid().ToString();
            var result = new InternalGenericResponse();
            HttpStatusCode statusCode = HttpStatusCode.InternalServerError;
            try
            {
                ChargePointService chargePointService = new ChargePointService();

                if (!chargePointService.IsOnline(ChargeBoxId))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CHARGEBOXID_ISNT_EXIST_OR_OFFLINE });
                }

                var connectorStatus = chargePointService.GetChargePointCurrentSatus(ChargeBoxId, 0);

                if (connectorStatus.HasValue && connectorStatus.Value == ChargePointStatus.Available)
                {
                    //create reqest

                    var request = new CancelReservationRequest()
                    {
                        reservationId = ReservationId

                    };

                    if (!AddCommandtoServer(ChargeBoxId, uuid, request))
                    {
                        throw new Exception("Write Command Fail!!");
                    }

                    statusCode = HttpStatusCode.Accepted;
                    result.SerialNo = uuid;
                }
                else
                {
                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CONNECTOR_ISNOT_AVAILIABLE_MODE });
                }


            }
            catch (Exception ex)
            {

                return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse()
                {
                    Message = string.Format(EVCBlConfiguration.ERRORMESSAGE_FORMAT, EVCBlConfiguration.ERROR_MSG_UNEXPECTEDERROR, "CancelResrvation", ex.ToString())
                });
            }

            return Request.CreateResponse(statusCode, result);
        }

        [Route("reset/{ResetType}/chargebox/{ChargeBoxId}")]
        [ResponseType(typeof(ErrorResponse))]
        [ResponseType(typeof(InternalGenericResponse))]
        [HttpPost]
        public HttpResponseMessage Reset(string ChargeBoxId, int ResetType)
        {
            string uuid = Guid.NewGuid().ToString();
            var result = new InternalGenericResponse();
            HttpStatusCode statusCode = HttpStatusCode.InternalServerError;
            try
            {
                ChargePointService chargePointService = new ChargePointService();

                if (!chargePointService.IsOnline(ChargeBoxId))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CHARGEBOXID_ISNT_EXIST_OR_OFFLINE });
                }

                var connectorStatus = chargePointService.GetChargePointCurrentSatus(ChargeBoxId, 0);

                if (connectorStatus.HasValue && connectorStatus.Value == ChargePointStatus.Available)
                {
                    //create reqest

                    var request = new ResetRequest()
                    {
                        type = (Packet.Messages.SubTypes.ResetType)ResetType

                    };

                    if (!AddCommandtoServer(ChargeBoxId, uuid, request))
                    {
                        throw new Exception("Write Command Fail!!");
                    }

                    statusCode = HttpStatusCode.Accepted;
                    result.SerialNo = uuid;
                }
                else
                {
                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CONNECTOR_ISNOT_AVAILIABLE_MODE });
                }


            }
            catch (Exception ex)
            {

                return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse()
                {
                    Message = string.Format(EVCBlConfiguration.ERRORMESSAGE_FORMAT, EVCBlConfiguration.ERROR_MSG_UNEXPECTEDERROR, "Reset", ex.ToString())
                });
            }

            return Request.CreateResponse(statusCode, result);
        }

        [Route("locallist/chargebox/{ChargeBoxId}")]
        [ResponseType(typeof(ErrorResponse))]
        [ResponseType(typeof(InternalGenericResponse))]
        [HttpPost]
        public HttpResponseMessage SendLocalList(string ChargeBoxId, [FromBody]LocalListRequest LocalList)
        {
            string uuid = Guid.NewGuid().ToString();
            var result = new InternalGenericResponse();
            HttpStatusCode statusCode = HttpStatusCode.InternalServerError;
            try
            {
                ChargePointService chargePointService = new ChargePointService();

                if (!chargePointService.IsOnline(ChargeBoxId))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CHARGEBOXID_ISNT_EXIST_OR_OFFLINE });
                }

                //check parameter
                if (LocalList == null ||
                    !IsValidEnum<UpdateType>((int)LocalList.UpdateType) || LocalList.LocalAuthorizationList == null
                    || (LocalList.LocalAuthorizationList != null && LocalList.LocalAuthorizationList.Count == 0))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_PARAMETER_OUTOFRANGE_INCORRECT });
                }

                bool badRequest = false;
                List<AuthorizationData> localAuthorizationList = new List<AuthorizationData>();
                for (int i = 0; i < LocalList.LocalAuthorizationList.Count; i++)
                {
                    if (string.IsNullOrEmpty(LocalList.LocalAuthorizationList[i].IdTag))
                        badRequest = true;

                    if (LocalList.LocalAuthorizationList[i].IdTagInfo != null && !IsValidEnum<AuthorizationStatus>((int)LocalList.LocalAuthorizationList[i].IdTagInfo.Status))
                        badRequest = true;

                    if (badRequest)
                    {
                        return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_PARAMETER_OUTOFRANGE_INCORRECT });
                    }
                    localAuthorizationList.Add(new AuthorizationData()
                    {
                        idTag = LocalList.LocalAuthorizationList[i].IdTag,
                        idTagInfo = LocalList.LocalAuthorizationList[i].IdTagInfo == null ? null :
                              new IdTagInfo()
                              {
                                  expiryDate = LocalList.LocalAuthorizationList[i].IdTagInfo.ExpiryDate,
                                  parentIdTag = LocalList.LocalAuthorizationList[i].IdTagInfo.ParentIdTag,
                                  status = (AuthorizationStatus)LocalList.LocalAuthorizationList[i].IdTagInfo.Status
                              }
                    });
                }

                var request = new SendLocalListRequest()
                {
                    listVersion = LocalList.ListVersion,
                    updateType = (UpdateType)LocalList.UpdateType,
                    localAuthorizationList = localAuthorizationList

                };

                if (!AddCommandtoServer(ChargeBoxId, uuid, request))
                {
                    throw new Exception("Write Command Fail!!");
                }

                statusCode = HttpStatusCode.Accepted;
                result.SerialNo = uuid;
            }
            catch (Exception ex)
            {

                return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse()
                {
                    Message = string.Format(EVCBlConfiguration.ERRORMESSAGE_FORMAT, EVCBlConfiguration.ERROR_MSG_UNEXPECTEDERROR, "ClearCache", ex.ToString())
                });
            }

            return Request.CreateResponse(statusCode, result);
        }


        [Route("cache/chargebox/{ChargeBoxId}")]
        [ResponseType(typeof(ErrorResponse))]
        [ResponseType(typeof(InternalGenericResponse))]
        [HttpDelete]
        public HttpResponseMessage ClearCache(string ChargeBoxId)
        {
            string uuid = Guid.NewGuid().ToString();
            var result = new InternalGenericResponse();
            HttpStatusCode statusCode = HttpStatusCode.InternalServerError;
            try
            {
                ChargePointService chargePointService = new ChargePointService();

                if (!chargePointService.IsOnline(ChargeBoxId))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CHARGEBOXID_ISNT_EXIST_OR_OFFLINE });
                }

                var connectorStatus = chargePointService.GetChargePointCurrentSatus(ChargeBoxId, 0);

                if (connectorStatus.HasValue && connectorStatus.Value == ChargePointStatus.Available)
                {

                    if (!AddCommandtoServer(ChargeBoxId, uuid, new ClearCacheRequest()))
                    {
                        throw new Exception("Write Command Fail!!");
                    }

                    statusCode = HttpStatusCode.Accepted;
                    result.SerialNo = uuid;
                }
                else
                {
                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CONNECTOR_ISNOT_AVAILIABLE_MODE });
                }


            }
            catch (Exception ex)
            {

                return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse()
                {
                    Message = string.Format(EVCBlConfiguration.ERRORMESSAGE_FORMAT, EVCBlConfiguration.ERROR_MSG_UNEXPECTEDERROR, "ClearCache", ex.ToString())
                });
            }

            return Request.CreateResponse(statusCode, result);
        }

        [Route("chargingprofile/chargebox/{ChargeBoxId}")]
        [ResponseType(typeof(ErrorResponse))]
        [ResponseType(typeof(InternalGenericResponse))]
        [HttpPost]
        public HttpResponseMessage SetChargingProfile(string ChargeBoxId, [FromBody]ChargingProfileRequest ChargingProfile)
        {

            string uuid = Guid.NewGuid().ToString();
            var result = new InternalGenericResponse();
            HttpStatusCode statusCode = HttpStatusCode.InternalServerError;
            try
            {
                ChargePointService chargePointService = new ChargePointService();

                if (!chargePointService.IsOnline(ChargeBoxId))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CHARGEBOXID_ISNT_EXIST_OR_OFFLINE });
                }

                //check parameter
                if (ChargingProfile == null ||
                    ChargingProfile.connectorId < 0 ||
                  !(ChargingProfile.ChargingProfile != null && IsValidEnum<ChargingProfileKindType>((int)ChargingProfile.ChargingProfile.chargingProfileKind)) ||
                  !(ChargingProfile.ChargingProfile != null && ChargingProfile.ChargingProfile.recurrencyKind.HasValue && IsValidEnum<RecurrencyKindType>((int)ChargingProfile.ChargingProfile.recurrencyKind)) ||
                  !(ChargingProfile.ChargingProfile != null && ChargingProfile.ChargingProfile.recurrencyKind.HasValue && IsValidEnum<RecurrencyKindType>((int)ChargingProfile.ChargingProfile.recurrencyKind)) ||
                  !(ChargingProfile.ChargingProfile != null && IsValidEnum<ChargingProfilePurposeType>((int)ChargingProfile.ChargingProfile.chargingProfilePurpose)) ||
                  !(ChargingProfile.ChargingProfile != null && ChargingProfile.ChargingProfile.chargingSchedule != null && IsValidEnum<ChargingRateUnitType>((int)ChargingProfile.ChargingProfile.chargingSchedule.chargingRateUnit)))
                {
                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_PARAMETER_OUTOFRANGE_INCORRECT });
                }

                var request = new SetChargingProfileRequest()
                {
                    connectorId = ChargingProfile.connectorId,
                    csChargingProfiles = new csChargingProfiles()
                    {
                        chargingProfileId = ChargingProfile.ChargingProfile.chargingProfileId,
                        chargingProfileKind = ChargingProfile.ChargingProfile.chargingProfileKind,
                        chargingProfilePurpose = ChargingProfile.ChargingProfile.chargingProfilePurpose,
                        recurrencyKind = ChargingProfile.ChargingProfile.recurrencyKind,
                        stackLevel = ChargingProfile.ChargingProfile.stackLevel,
                        transactionId = ChargingProfile.ChargingProfile.TransactionId,
                        validFrom = ChargingProfile.ChargingProfile.validFrom == null ? (DateTime?)null : DateTime.SpecifyKind(ChargingProfile.ChargingProfile.validFrom.Value, DateTimeKind.Utc),
                        validTo = ChargingProfile.ChargingProfile.validTo == null ? (DateTime?)null : DateTime.SpecifyKind(ChargingProfile.ChargingProfile.validTo.Value, DateTimeKind.Utc),
                        chargingSchedule = new ChargingSchedule()
                        {
                            chargingRateUnit = ChargingProfile.ChargingProfile.chargingSchedule.chargingRateUnit,
                            chargingSchedulePeriod = ChargingProfile.ChargingProfile.chargingSchedule.chargingSchedulePeriod,
                            duration = ChargingProfile.ChargingProfile.chargingSchedule.duration,
                            minChargingRate = ChargingProfile.ChargingProfile.chargingSchedule.minChargingRate,
                            startSchedule = ChargingProfile.ChargingProfile.chargingSchedule.startSchedule == null ? (DateTime?)null : DateTime.SpecifyKind(ChargingProfile.ChargingProfile.chargingSchedule.startSchedule.Value, DateTimeKind.Utc)
                        }
                    }
                };


                if (!AddCommandtoServer(ChargeBoxId, uuid, request))
                {
                    throw new Exception("Write Command Fail!!");
                }


                statusCode = HttpStatusCode.Accepted;
                result.SerialNo = uuid;

            }
            catch (Exception ex)
            {

                return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse()
                {
                    Message = string.Format(EVCBlConfiguration.ERRORMESSAGE_FORMAT, EVCBlConfiguration.ERROR_MSG_UNEXPECTEDERROR, "StopTransaction", ex.ToString())
                });
            }

            return Request.CreateResponse(statusCode, result);
        }

        [Route("chargeprofile/chargebox/{ChargeBoxId}")]
        [ResponseType(typeof(ErrorResponse))]
        [ResponseType(typeof(InternalGenericResponse))]
        [HttpDelete]
        public HttpResponseMessage ClearChargingProfile(string ChargeBoxId, int Id = -1, int ConnectorId = -1, int ChargingProfilePurpose = -1, int StackLevel = -1)
        {
            string uuid = Guid.NewGuid().ToString();
            var result = new InternalGenericResponse();
            HttpStatusCode statusCode = HttpStatusCode.InternalServerError;
            try
            {
                ChargePointService chargePointService = new ChargePointService();

                if (!chargePointService.IsOnline(ChargeBoxId))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CHARGEBOXID_ISNT_EXIST_OR_OFFLINE });
                }

                var connectorStatus = chargePointService.GetChargePointCurrentSatus(ChargeBoxId, 0);

                if (connectorStatus.HasValue && connectorStatus.Value == ChargePointStatus.Available)
                {

                    var request = new ClearChargingProfileRequest()
                    {
                        chargingProfilePurpose = ChargingProfilePurpose == -1 ? (ChargingProfilePurposeType?)null : (ChargingProfilePurposeType)ChargingProfilePurpose,
                        connectorId = ConnectorId == -1 ? (int?)null : (int)ConnectorId,
                        id = Id == -1 ? (int?)null : (int)ChargingProfilePurpose,
                        stackLevel = StackLevel == -1 ? (int?)null : (int)StackLevel
                    };

                    if (!AddCommandtoServer(ChargeBoxId, uuid, request))
                    {
                        throw new Exception("Write Command Fail!!");
                    }

                    statusCode = HttpStatusCode.Accepted;
                    result.SerialNo = uuid;
                }
                else
                {
                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CONNECTOR_ISNOT_AVAILIABLE_MODE });
                }


            }
            catch (Exception ex)
            {

                return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse()
                {
                    Message = string.Format(EVCBlConfiguration.ERRORMESSAGE_FORMAT, EVCBlConfiguration.ERROR_MSG_UNEXPECTEDERROR, "ClearChargingProfile", ex.ToString())
                });
            }

            return Request.CreateResponse(statusCode, result);
        }

        [Route("trigger/chargebox/{ChargeBoxId}")]
        [ResponseType(typeof(ErrorResponse))]
        [ResponseType(typeof(InternalGenericResponse))]
        [HttpPost]
        public HttpResponseMessage TriggerMessage(string ChargeBoxId, [FromBody] TriggerRequest triggerMessage)
        {
            string uuid = Guid.NewGuid().ToString();
            var result = new InternalGenericResponse();
            HttpStatusCode statusCode = HttpStatusCode.InternalServerError;
            try
            {
                ChargePointService chargePointService = new ChargePointService();

                if (!chargePointService.IsOnline(ChargeBoxId))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CHARGEBOXID_ISNT_EXIST_OR_OFFLINE });
                }

                //check parameter
                if (triggerMessage == null || !(triggerMessage.TriggerType >= 1 && triggerMessage.TriggerType <= (int)MessageTrigger.StatusNotification))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_PARAMETER_OUTOFRANGE_INCORRECT });
                }

                //create reqest

                var request = new TriggerMessageRequest()
                {
                    connectorId = triggerMessage.ConnectorId == -1 ? (int?)null : triggerMessage.ConnectorId,
                    requestedMessage = (MessageTrigger)triggerMessage.TriggerType
                };


                if (!AddCommandtoServer(ChargeBoxId, uuid, request))
                {
                    throw new Exception("Write Command Fail!!");
                }

                statusCode = HttpStatusCode.Accepted;
                result.SerialNo = uuid;

            }
            catch (Exception ex)
            {

                return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse()
                {
                    Message = string.Format(EVCBlConfiguration.ERRORMESSAGE_FORMAT, EVCBlConfiguration.ERROR_MSG_UNEXPECTEDERROR, "TriggerMessage", ex.ToString())
                });
            }

            return Request.CreateResponse(statusCode, result);
        }

        [Route("unlockconnector/chargebox/{ChargeBoxId}/ connectorId/{ConnectorId}")]
        [ResponseType(typeof(ErrorResponse))]
        [ResponseType(typeof(InternalGenericResponse))]
        [HttpPost]
        public HttpResponseMessage UnlockConnector(string ChargeBoxId, int ConnectorId)
        {
            string uuid = Guid.NewGuid().ToString();
            var result = new InternalGenericResponse();
            HttpStatusCode statusCode = HttpStatusCode.InternalServerError;
            try
            {
                ChargePointService chargePointService = new ChargePointService();

                if (!chargePointService.IsOnline(ChargeBoxId))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CHARGEBOXID_ISNT_EXIST_OR_OFFLINE });
                }

                //check parameter
                if (ConnectorId < 0)
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_PARAMETER_OUTOFRANGE_INCORRECT });
                }

                //create reqest

                var request = new UnlockConnectorRequest()
                {
                    connectorId = ConnectorId
                };


                if (!AddCommandtoServer(ChargeBoxId, uuid, request))
                {
                    throw new Exception("Write Command Fail!!");
                }

                statusCode = HttpStatusCode.Accepted;
                result.SerialNo = uuid;

            }
            catch (Exception ex)
            {

                return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse()
                {
                    Message = string.Format(EVCBlConfiguration.ERRORMESSAGE_FORMAT, EVCBlConfiguration.ERROR_MSG_UNEXPECTEDERROR, "UnlockConnector", ex.ToString())
                });
            }

            return Request.CreateResponse(statusCode, result);
        }


        [Route("compositeschedule/chargebox/{ChargeBoxId}")]
        [ResponseType(typeof(ErrorResponse))]
        [ResponseType(typeof(InternalGenericResponse))]
        [HttpGet]
        public HttpResponseMessage CompositeSchedule(string ChargeBoxId, int ConnectorId, int Duration, int ChargingRateUnit = -1)
        {
            string uuid = Guid.NewGuid().ToString();
            var result = new InternalGenericResponse();
            HttpStatusCode statusCode = HttpStatusCode.InternalServerError;
            try
            {

                ChargePointService chargePointService = new ChargePointService();

                if (!chargePointService.IsOnline(ChargeBoxId))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_CHARGEBOXID_ISNT_EXIST_OR_OFFLINE });
                }

                //check parameter
                if (!(ConnectorId >= 0) ||
                    !(ChargingRateUnit != -1 && ChargingRateUnit > 0 && ChargingRateUnit < 3))
                {

                    return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = EVCBlConfiguration.ERROR_MSG_PARAMETER_OUTOFRANGE_INCORRECT });
                }


                var request = new GetCompositeScheduleRequest()
                {
                    connectorId = ConnectorId,
                    duration = Duration,
                    chargingRateUnit = ChargingRateUnit > 0 ? (ChargingRateUnitType)ChargingRateUnit : (ChargingRateUnitType?)null
                };

                if (!AddCommandtoServer(ChargeBoxId, uuid, request))
                {
                    throw new Exception("Write Command Fail!!");
                }

                statusCode = HttpStatusCode.Accepted;
                result.SerialNo = uuid;

            }
            catch (Exception ex)
            {

                return Request.CreateResponse(HttpStatusCode.BadRequest, new ErrorResponse() { Message = string.Format(EVCBlConfiguration.ERRORMESSAGE_FORMAT, EVCBlConfiguration.ERROR_MSG_UNEXPECTEDERROR, "GetLocallistversion", ex.ToString()) });
            }

            return Request.CreateResponse(statusCode, result);
        }


        private bool AddCommandtoServer(string chargeBoxId, string uuid, IRequest request)
        {
            bool result = false;
            try
            {
                ServerTriggerService service = new ServerTriggerService();
                service.AddMessage(chargeBoxId, uuid, request);
                result = true;
            }
            catch (Exception ex)
            {
                ;
            }

            return result;
        }


        private bool IsValidEnum<TEnum>(int enumValue)
        {

            return Enum.IsDefined(typeof(TEnum), enumValue);

        }

    }
}
